/************************************************************************
 *   IRC - Internet Relay Chat, ircd/s_externs.h
 *   Copyright (C) 1997 Alain Nissen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*  This file includes all *_ext.h files containing external declarations
 *  for the IRC server.
 */

#include "bsd_ext.h"
#include "channel_ext.h"
#include "class_ext.h"
#include "dbuf_ext.h"
#include "hash_ext.h"
#include "ircd_ext.h"
#include "list_ext.h"
#include "match_ext.h"
#include "packet_ext.h"
#include "parse_ext.h"
#include "res_comp_ext.h"
#include "res_ext.h"
#include "res_init_ext.h"
#include "res_mkquery_ext.h"
#include "s_auth_ext.h"
#include "s_bsd_ext.h"
#include "s_conf_ext.h"
#include "s_debug_ext.h"
#include "s_err_ext.h"
#include "s_misc_ext.h"
#include "s_numeric_ext.h"
#include "s_serv_ext.h"
#include "s_service_ext.h"
#include "s_send_ext.h"
#include "s_user_ext.h"
#include "s_zip_ext.h"
#include "s_id_ext.h"
#include "send_ext.h"
#include "support_ext.h"
#include "version_ext.h"
#include "whowas_ext.h"
