/************************************************************************
 *   IRC - Internet Relay Chat, common/msg_def.h
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define MSG_PRIVATE  "PRIVMSG"
#define MSG_WHO      "WHO"
#define MSG_WHOIS    "WHOIS"
#define MSG_WHOWAS   "WHOWAS"
#define MSG_USER     "USER"
#define MSG_NICK     "NICK"
#define MSG_UNICK    "UNICK"
#define MSG_SERVER   "SERVER"
#define MSG_LIST     "LIST"
#define MSG_TOPIC    "TOPIC"
#define MSG_INVITE   "INVITE"
#define MSG_VERSION  "VERSION"
#define MSG_QUIT     "QUIT"
#define MSG_SQUIT    "SQUIT"
#define MSG_KILL     "KILL"
#define MSG_SAVE     "SAVE"
#define MSG_INFO     "INFO"
#define MSG_LINKS    "LINKS"
#define MSG_SUMMON   "SUMMON"
#define MSG_STATS    "STATS"
#define MSG_USERS    "USERS"
#define MSG_HELP     "HELP"
#define MSG_ERROR    "ERROR"
#define MSG_AWAY     "AWAY"
#define MSG_CONNECT  "CONNECT"
#define MSG_PING     "PING"
#define MSG_PONG     "PONG"
#define MSG_OPER     "OPER"
#define MSG_PASS     "PASS"
#define MSG_WALLOPS  "WALLOPS"
#define MSG_TIME     "TIME"
#define MSG_NAMES    "NAMES"
#define MSG_ADMIN    "ADMIN"
#define MSG_TRACE    "TRACE"
#define MSG_NOTICE   "NOTICE"
#define MSG_JOIN     "JOIN"
#define MSG_NJOIN    "NJOIN"
#define MSG_PART     "PART"
#define MSG_LUSERS   "LUSERS"
#define MSG_MOTD     "MOTD"
#define MSG_MODE     "MODE"
#define MSG_UMODE    "UMODE"
#define MSG_KICK     "KICK"
#define MSG_SERVICE  "SERVICE"
#define MSG_USERHOST "USERHOST"
#define MSG_ISON     "ISON"
#define MSG_NOTE     "NOTE"
#define MSG_SQUERY   "SQUERY"
#define MSG_SERVLIST "SERVLIST"
#define MSG_SERVSET  "SERVSET"
#define	MSG_REHASH   "REHASH"
#define	MSG_RESTART  "RESTART"
#define	MSG_CLOSE    "CLOSE"
#define	MSG_DIE	     "DIE"
#define	MSG_HASH     "HAZH"
#define	MSG_DNS      "DNS"
#define	MSG_SMASK    "SMASK"
#define MSG_EOB	     "EOB"
#define MSG_SET      "SET"
#define MSG_MAP	     "MAP"
#define MSG_POST     "POST"

#define MAXPARA		15

